/* $Id: option.c,v 1.7 1998/03/16 05:45:45 svmcguir Exp $ */

#include <stdlib.h>
#include <math.h>
#include "path.h"
#include "option.h"
#include "util.h"

void optionDataAllocate(optionData *od){
  /* Allocates od.payoff[0:time-1][0:path-1] using od.pData.times
   * and od.params.pData.paths.
   */

  allocate2dDouble(&(od->payoff),od->pData.times,od->pData.meshPaths);
  allocate2dDouble(&(od->euroPrice),od->pData.times,od->pData.meshPaths);
  allocate2dInt(&(od->euroChoice),od->pData.times,od->pData.meshPaths);
}

void optionDataDeallocate(optionData *od){
  /* Deallocates od.payoff[0:time-1][0:path-1] using od.pData.times
   * and od.params.pData.paths.
   */

  deallocate2dDouble(&(od->payoff),od->pData.times,od->pData.meshPaths);
  deallocate2dDouble(&(od->euroPrice),od->pData.times,od->pData.meshPaths);
  deallocate2dInt(&(od->euroChoice),od->pData.times,od->pData.meshPaths);
}

double optionPayoff(optionData *od, double *price){
  /* Returns the payoff on the option described by od for the prices in
   * the array price.  The dimension of price is [0:od.underliers-1].
   * This is written for a max option.
   */

  double max,min;

  /* For a max (call) */

  max = dArrayMax(od->pData.underliers,price);
  return(max - od->strikePrice);


  /* For a min (put) */
  /*
    min = dArrayMin(od->pData.underliers,price); 
    return(od->strikePrice - min);
  */
}


void optionDataFill(optionData *od){
  /* Fills od->payoff with payoffs for the underlier prices found in pd.
   * Also, deflates them with the risk free rate.
   */

  int time, path, choice;

  for(time=0; time < od->pData.times; time++){
    for(path=0; path < od->pData.meshPaths; path++){
      od->payoff[time][path] = exp(-time*od->riskFreeRate)*
        optionPayoff(od,od->pData.price[time][path]);
    }
  }

}

